"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const Websocket = require("ws");
const paho_mqtt_1 = require("paho-mqtt");
global.WebSocket = Websocket;
class GQLSubscription {
    constructor(selection, url, clientId, topics, runner) {
        this.messages = [];
        this.subscribers = [];
        this.subscriberMessages = {};
        this.notifySubcribers = (result) => {
            this.subscribers.forEach(({ id, matches, onMatch }) => {
                if (matches(result)) {
                    if (!this.subscriberMessages[id]) {
                        this.subscriberMessages[id] = [];
                    }
                    this.subscriberMessages[id].push(result);
                    onMatch.forEach(fn => fn(result));
                }
            });
        };
        this.addListener = (listenerId, matcher) => {
            this.subscribers.push({
                id: listenerId,
                matches: (message) => {
                    try {
                        chai_1.expect(message).to.containSubset(matcher);
                        return true;
                    }
                    catch (error) {
                        return false;
                    }
                },
                onMatch: [],
            });
            this.messages.forEach(message => this.notifySubcribers(message));
        };
        this.disconnect = () => {
            this.client.disconnect();
        };
        this.listenerMessage = async (listenerId, timeout = 5000) => {
            if (this.subscriberMessages[listenerId] &&
                this.subscriberMessages[listenerId].length) {
                return this.subscriberMessages[listenerId][this.subscriberMessages[listenerId].length - 1];
            }
            let messageListenerId;
            return new Promise((resolve, reject) => {
                const sub = this.subscribers.find(({ id }) => id === listenerId);
                if (!sub) {
                    throw new Error(`Subscriber for "${listenerId}" not found!`);
                }
                const timeoutId = setTimeout(() => {
                    sub.onMatch.splice(messageListenerId, 1);
                    reject();
                }, timeout);
                messageListenerId = sub.onMatch.push((msg) => {
                    clearTimeout(timeoutId);
                    resolve(msg);
                });
            });
        };
        this.client = new paho_mqtt_1.Client(url, clientId);
        this.client.onMessageArrived = (msg) => {
            const { data: { [selection]: result }, } = JSON.parse(msg.payloadString);
            this.messages.push(result);
            runner.progress('<GQL@', msg.payloadString).catch(() => { });
            this.notifySubcribers(result);
        };
        this.connection = new Promise((resolve, reject) => {
            this.client.connect({
                useSSL: false,
                mqttVersion: 3,
                onSuccess: async () => {
                    await Promise.all(topics.map(topic => new Promise((resolve1, reject1) => {
                        this.client.subscribe(topic, {
                            onSuccess: resolve1,
                            onFailure: reject1,
                        });
                    })));
                    resolve(this.client);
                },
                onFailure: reject,
            });
        });
    }
}
exports.GQLSubscription = GQLSubscription;
