"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const AWS = require("aws-sdk");
const aws_sdk_1 = require("aws-sdk");
const node_fetch_1 = require("node-fetch");
const url_1 = require("url");
const parseQuery_1 = require("./parseQuery");
exports.queryWithIAM = (AccessKeyId, SecretKey, SessionToken, endpoint) => async (gqlQuery, variables) => {
    const credentials = new aws_sdk_1.Credentials(AccessKeyId, SecretKey, SessionToken);
    const { selection, operation } = parseQuery_1.parseQuery(gqlQuery);
    const graphQLEndpoint = url_1.parse(endpoint);
    const region = graphQLEndpoint.host.split('.')[2];
    const httpRequest = new aws_sdk_1.HttpRequest(new aws_sdk_1.Endpoint(endpoint), region);
    const query = {
        query: gqlQuery,
        variables,
    };
    httpRequest.headers.host = graphQLEndpoint.host;
    httpRequest.headers['Content-Type'] = 'application/json';
    httpRequest.method = 'POST';
    httpRequest.region = region;
    httpRequest.body = JSON.stringify(query);
    const signer = new AWS.Signers.V4(httpRequest, 'appsync', true);
    signer.addAuthorization(credentials, new Date());
    const options = {
        method: httpRequest.method,
        body: httpRequest.body,
        headers: httpRequest.headers,
    };
    const response = await node_fetch_1.default(graphQLEndpoint.href, options);
    return {
        operation,
        selection,
        result: await response.json(),
    };
};
