"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const runner_1 = require("./runner");
const console_reporter_1 = require("./console-reporter");
const storage_1 = require("../steps/storage");
const createRunner = (config) => {
    const mockConsole = {
        log: jest.fn(),
        error: jest.fn(),
    };
    const runner = new runner_1.FeatureRunner({}, {
        reporters: [
            new console_reporter_1.ConsoleReporter({
                ...config,
                console: mockConsole,
            }),
        ],
        dir: path.join(process.cwd(), 'test', 'storage'),
        retry: false,
        store: {
            foo: 'bar',
            bar: {
                baz: 'foo',
                num: 42,
                b: true,
            },
        },
    });
    return {
        runner,
        mockConsole,
    };
};
const stripColors = (s) => s.replace(/\u001b\[[0-9;]+m/g, '');
describe('Console Reporter', () => {
    it('should print a summary if printSummary=true', async () => {
        const { runner, mockConsole } = createRunner({ printSummary: true });
        const { success } = await runner.addStepRunners(storage_1.storageStepRunners()).run();
        expect(success).toEqual(true);
        const logs = mockConsole.log.mock.calls
            .map(args => args.join(' '))
            .join('\n');
        expect(stripColors(logs)).toMatch(/Feature Summary:   0 failed, 0 skipped, 3 passed, 3 total/);
        expect(stripColors(logs)).toMatch(/Scenario Summary:  0 failed, 0 skipped, 10 passed, 10 total/);
    });
    it('should not print a summary if printSummary=false', async () => {
        const { runner, mockConsole } = createRunner();
        const { success } = await runner.addStepRunners(storage_1.storageStepRunners()).run();
        expect(success).toEqual(true);
        const logs = mockConsole.log.mock.calls
            .map(args => args.join(' '))
            .join('\n');
        expect(logs).not.toContain('Feature Summary:');
    });
});
