"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const runner_1 = require("./runner");
const TokenScanner_1 = require("gherkin/dist/src/TokenScanner");
const AstBuilder_1 = require("gherkin/dist/src/AstBuilder");
const cucumber_messages_1 = require("cucumber-messages");
const Parser_1 = require("gherkin/dist/src/Parser");
const TokenMatcher_1 = require("gherkin/dist/src/TokenMatcher");
const toposort = require("toposort");
const globAsync = require("glob");
const util_1 = require("util");
const path = require("path");
const glob = util_1.promisify(globAsync);
const fs_1 = require("fs");
const parser = new Parser_1.default(new AstBuilder_1.default(cucumber_messages_1.IdGenerator.uuid()));
const matcher = new TokenMatcher_1.default();
exports.parseFeatures = (featureData) => {
    const parsedFeatures = featureData.map(d => {
        const scanner = new TokenScanner_1.default(d.toString());
        return parser.parse(scanner, matcher)
            .feature;
    });
    const sortedByLast = parsedFeatures.sort(({ tags: t1 }) => (t1 || []).find(({ name }) => name === '@Last') ? 1 : -1);
    const featureNames = sortedByLast.map(({ name }) => name);
    const featureDependencies = sortedByLast.map(feature => {
        var _a;
        const bgSteps = (_a = feature.children) === null || _a === void 0 ? void 0 : _a.filter(({ background }) => background).map(bg => {
            var _a;
            return (((_a = bg.background) === null || _a === void 0 ? void 0 : _a.steps) || []).filter(({ text }) => text && runner_1.afterRx.test(text));
        }).flat();
        const runAfter = bgSteps === null || bgSteps === void 0 ? void 0 : bgSteps.map(afterStep => {
            const m = afterStep.text && runner_1.afterRx.exec(afterStep.text);
            if (!m) {
                throw new Error(`Failed to find feature in ${afterStep.text}`);
            }
            if (!featureNames.includes(m[1])) {
                throw new Error(`The feature ${m[1]} you want to run after does not exist!`);
            }
            return m[1];
        });
        if (runAfter === null || runAfter === void 0 ? void 0 : runAfter.length) {
            return runAfter.map(dep => [dep, feature.name]);
        }
        return [[feature.name, undefined]];
    });
    const sortedFeatureNames = toposort(featureDependencies.flat()).filter((feature) => feature);
    const dependencies = (f) => sortedFeatures.filter(({ name }) => {
        const depNames = featureDependencies
            .flat()
            .filter(([, fname]) => fname === f.name)
            .map(([depName]) => depName);
        return depNames.includes(name);
    });
    const sortedFeatures = sortedFeatureNames.map((featureName) => parsedFeatures.find(({ name }) => name === featureName));
    const isOnly = (f) => { var _a; return (_a = f === null || f === void 0 ? void 0 : f.tags) === null || _a === void 0 ? void 0 : _a.find(({ name }) => name === '@Only'); };
    const only = parsedFeatures.filter(isOnly);
    const onlyNames = only.map(({ name }) => name);
    return sortedFeatures.map(f => {
        const { tags, name: featureName } = f;
        const skip = (tags || []).find(({ name }) => name === '@Skip') ||
            (onlyNames.length && !onlyNames.includes(featureName));
        return {
            ...f,
            skip: !!skip,
            dependsOn: dependencies(f),
        };
    });
};
exports.fromDirectory = async (dir) => {
    const scan = path.join(path.resolve(dir), '*.feature');
    const featureFiles = await glob(scan);
    const features = exports.parseFeatures(featureFiles.map(f => fs_1.readFileSync(f)));
    if (!features.length) {
        throw new Error(`No features found in directory ${dir}`);
    }
    return features;
};
