"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const aws_sdk_1 = require("aws-sdk");
class WebhookReceiver {
    constructor(queueUrl, region) {
        this.queueUrl = queueUrl;
        this.sqs = new aws_sdk_1.SQS({ region });
    }
    async receiveWebhookRequest(MessageGroupId, runner) {
        const { Messages } = await this.sqs
            .receiveMessage({
            QueueUrl: this.queueUrl,
            MaxNumberOfMessages: 1,
            MessageAttributeNames: ['All'],
            AttributeNames: ['MessageGroupId'],
            WaitTimeSeconds: 20,
        })
            .promise();
        if (Messages === undefined || !Messages.length) {
            throw new Error('No webhook request received!');
        }
        const { Body, MessageAttributes, ReceiptHandle, Attributes } = Messages[0];
        await this.sqs
            .deleteMessage({
            QueueUrl: this.queueUrl,
            ReceiptHandle: ReceiptHandle,
        })
            .promise();
        const attrs = MessageAttributes;
        const { MessageGroupId: RcvdMessageGroupId, } = Attributes;
        this.latestWebhookRequest = {
            headers: Object.keys(attrs || {}).reduce((hdrs, key) => {
                hdrs[key] = attrs[key].StringValue;
                return hdrs;
            }, {}),
            body: JSON.parse(Body),
        };
        await runner.progress(`Webhook < ${RcvdMessageGroupId}`, JSON.stringify(this.latestWebhookRequest.body));
        if (RcvdMessageGroupId !== MessageGroupId) {
            throw new Error(`Wrong webhook request received! Expected "${MessageGroupId}", got "${RcvdMessageGroupId}"`);
        }
        return this.latestWebhookRequest;
    }
    async clearQueue() {
        const { Messages } = await this.sqs
            .receiveMessage({
            QueueUrl: this.queueUrl,
            MaxNumberOfMessages: 10,
            WaitTimeSeconds: 0,
        })
            .promise();
        if (Messages !== undefined) {
            await Promise.all(Messages.map(async ({ ReceiptHandle }) => this.sqs
                .deleteMessage({
                QueueUrl: this.queueUrl,
                ReceiptHandle: ReceiptHandle,
            })
                .promise()));
            await this.clearQueue();
            this.latestWebhookRequest = undefined;
        }
    }
}
exports.WebhookReceiver = WebhookReceiver;
