"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsonata = require("jsonata");
const chai = require("chai");
const chai_1 = require("chai");
const regexGroupMatcher_1 = require("../lib/regexGroupMatcher");
const chaiSubset = require("chai-subset");
chai.use(chaiSubset);
exports.storageStepRunners = () => [
    regexGroupMatcher_1.regexGroupMatcher(/^"(?<exp>[^"]+)" should (?<equalOrMatch>(?:equal|be)|match) (?:(?<jsonMatch>this JSON)|"(?<stringMatch>[^"]+)"|(?<numMatch>[0-9]+)|(?<boolMatch>true|false))$/)(async ({ exp, equalOrMatch, jsonMatch, stringMatch, numMatch, boolMatch }, step, runner) => {
        let expected;
        if (jsonMatch) {
            if (!step.interpolatedArgument) {
                throw new Error('Must provide argument!');
            }
            expected = JSON.parse(step.interpolatedArgument);
        }
        else if (stringMatch) {
            expected = stringMatch;
        }
        else if (numMatch) {
            expected = parseInt(numMatch, 10);
        }
        else if (boolMatch) {
            expected = boolMatch === 'true';
        }
        const fragment = jsonata(exp).evaluate(runner.store);
        if (equalOrMatch === 'match') {
            chai_1.expect(fragment).to.containSubset(expected);
        }
        else {
            chai_1.expect(fragment).to.deep.equal(expected);
        }
        return [fragment];
    }),
    regexGroupMatcher_1.regexGroupMatcher(/^I escape this JSON into "(?<storeName>[^"]+)"$/)(async ({ storeName }, step, runner) => {
        if (!step.interpolatedArgument) {
            throw new Error('Must provide argument!');
        }
        const j = JSON.parse(step.interpolatedArgument);
        runner.store[storeName] = JSON.stringify(JSON.stringify(j));
        return runner.store[storeName];
    }),
    regexGroupMatcher_1.regexGroupMatcher(/^I parse "(?<exp>[^"]+)" into "(?<storeName>[^"]+)"$/)(async ({ exp, storeName }, _, runner) => {
        const e = jsonata(exp);
        const result = e.evaluate(runner.store);
        chai_1.expect(result).to.not.be.an('undefined');
        runner.store[storeName] = JSON.parse(result);
        return runner.store[storeName];
    }),
    regexGroupMatcher_1.regexGroupMatcher(/^I store "(?<exp>[^"]+)" into "(?<storeName>[^"]+)"$/)(async ({ exp, storeName }, _, runner) => {
        const e = jsonata(exp);
        const result = e.evaluate(runner.store);
        chai_1.expect(result).to.not.be.an('undefined');
        runner.store[storeName] = result;
        return result;
    }),
];
