"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jsonata = require("jsonata");
const chai = require("chai");
const chai_1 = require("chai");
const regexMatcher_1 = require("../lib/regexMatcher");
const webhook_receiver_1 = require("../lib/webhook-receiver");
const chaiSubset = require("chai-subset");
const regexGroupMatcher_1 = require("../lib/regexGroupMatcher");
chai.use(chaiSubset);
let r;
exports.webhookStepRunners = ({ region, }) => [
    regexMatcher_1.regexMatcher(/^the Webhook Receiver "([^"]+)" should be called$/)(async ([MessageGroupId], _, runner) => r.receiveWebhookRequest(MessageGroupId, runner).then(r => r.body)),
    regexMatcher_1.regexMatcher(/^"([^"]+)" of the webhook request body should equal "([^"]+)"$/)(async ([exp, expected]) => {
        var _a;
        const e = jsonata(exp);
        chai_1.expect(r.latestWebhookRequest).not.to.be.an('undefined');
        const b = (_a = r.latestWebhookRequest) === null || _a === void 0 ? void 0 : _a.body;
        chai_1.expect(b).not.to.be.an('undefined');
        const result = e.evaluate(b);
        chai_1.expect(result).to.deep.equal(expected);
        return b;
    }),
    regexMatcher_1.regexMatcher(/^the webhook request body should (equal|match) this JSON$/)(async ([equalOrMatch], step) => {
        var _a;
        if (!step.interpolatedArgument) {
            throw new Error('Must provide argument!');
        }
        const j = JSON.parse(step.interpolatedArgument);
        const b = (_a = r.latestWebhookRequest) === null || _a === void 0 ? void 0 : _a.body;
        chai_1.expect(b).not.to.be.an('undefined');
        if (equalOrMatch === 'match') {
            chai_1.expect(b).to.containSubset(j);
        }
        else {
            chai_1.expect(b).to.deep.equal(j);
        }
        return b;
    }),
    regexMatcher_1.regexMatcher(/^I have a Webhook Receiver/)(async (_, __, runner) => {
        r = new webhook_receiver_1.WebhookReceiver(runner.world.webhookQueue, region);
        await r.clearQueue();
    }),
    regexGroupMatcher_1.regexGroupMatcher(/^I store "(?<exp>[^"]+)" of the last webhook request into "(?<storeName>[^"]+)"$/)(async ({ exp, storeName }, _, runner) => {
        const e = jsonata(exp);
        const result = e.evaluate(r.latestWebhookRequest);
        chai_1.expect(result).to.not.be.an('undefined');
        runner.store[storeName] = result;
        return result;
    }),
];
